integrateMC <- function (func, from = 0, to = 1, n = 10000, graphical = FALSE) 
{
    ##### Verificando consistência de dados #####
	
    # Na primeira etapa, iremos avaliar se o argumento 'func'
    # realmente corresponde a uma função (Consistência de dados).

    # Primeiro, utilizamos a função 'substitute' para gerar uma
    # árvore de análise sintática da expressão contida em 'func'.	

    sfunc <- substitute(func)

    if (sfunc == substitute(x))
            sfunc = substitute(1*x)

    # A etapa anterior é importante para realizar a validação
    # da expressão contida em 'func'. Na condição seguinte,
    # caso 'sfunc' não contenha uma expressão válida ou
    # caso a expressão contida em 'func' contenha uma variável
    # diferente de 'x', nossa função 'Montecarlo' levanta uma
    # mensagem de erro, indicando que a expressão fornecida não 
    # é uma expressão válida.

  
    if (!(is.call(sfunc) && match("x", all.vars(sfunc), nomatch = 0L))) 
            stop("invalid function")
    

    ##### Fim da verificação da consistência de dados #####


    

    ##### Cálculo do Máximo e Mínimo da função 'func' #####

    # Nesta etapa, iremos encontrar o máximo e o mínimo
    # da função matemática 'func', dentro do intervalo
    # [from,to].

    max <- -Inf
    min <- Inf
	
    for(i in c(0:n)) {
	x <- from + i*(to-from)/n	
	y <- eval(sfunc)
	if(y>max)
		max<-y
	if(y<min)
		min<-y
    }

    if(min>0)
	min=0
    if(max<0)
	max=0   

    ##### Fim do cálculo de Máximo e Mínimo da função 'func' #####
 
    # O trecho a seguir plota a curva que representa a função 'func'	

    if(graphical)
    {	
    	x<-seq.int(from, to, length.out = 1000)
    	y <- eval(sfunc, envir = list(x = x), enclos = parent.frame())
    	plot(x, y, type = "l", ylim = c(min,max))
	curve(0*x,add=TRUE)  #Esta linha plota o Eixo x
    	invisible(list(x = x, y = y))
    }




    ##### Início do cálculo de probabilidades utilizando o método de Monte Carlo #####

    pontospos=0
    pontosneg=0

    for(i in c(0:n)) {

	#Gera um par de pontos aleatórios
	x <- runif(1,from,to)
	yrand <- runif(1,min,max)

	#Avalia a função no x aleatório encontrado
	y <-eval(sfunc)
	
	#Esta variável irá sinalizar se devemos plotar o ponto ou não	
	flag=1	


	#Se o ponto está entre a curva da função e o eixo x e é um ponto positivo
	if(yrand>0 && yrand<y)
	{
		#Aumentamos nosso contador de pontos que compreendem a região positiva
		
	 	pontospos<-pontospos+1
		

		# O trecho a seguir plota o ponto e indica que ele não precisa mais ser plottado
		flag=0		
		if(graphical)
			points(x,yrand,pch=20,col="red")
	}

	#Se o ponto está entre a curva da função e o eixo x, mas é negativo
	if(y<yrand && yrand < 0)
	{
		#Aumentamos nosso contador de pontos que compreendem a região negativa
		
		pontosneg<-pontosneg+1

		# O trecho a seguir plotta o ponto e indica que ele não precisa mais ser plottado
		flag=0
		if(graphical)
			points(x,yrand,pch=20,col="red")
	}
	
	# Caso o ponto ainda não tenha sido plottado, plottamos o ponto com outra cor 
	# para indicar que ele é um ponto fora da área da curva.

	if(flag==1 && graphical)
		points(x,yrand,pch=20,col="blue")
    }

    # A probabilidade de um ponto aleatório ser sorteado entre 
    # a parte positiva da curva e o eixo x é aproximadamente de:
	
    ProbPositiva = pontospos/n

    # A probabilidade de um ponto aleatório ser sorteado entre 
    # a parte negativa da curva e o eixo x é aproximadamente de:

    ProbNegativa = pontosneg/n

    # A área total do retângulo que compreende o nosso intervalo
    # de intregação e os extremos da função nesse intervalo é de:

    Areatotal <- (to-from)*(max-min)

    # A integral pode então ser estimada por:

    Integral <- Areatotal*ProbPositiva -Areatotal*ProbNegativa

    # O erro pode ser estimado por:

    Erro <- 1/sqrt(n)	
 
    ##### Fim do cálculo da Integral utilizando o Método de Monte Carlo #####

    ##### Impressão dos resultados encontrados: #####

    print(c("Estimated value:", Integral))
    print(c("Error:", Erro))

    return(c(Integral,Erro))

}
